/* Portions Copyright (c) 2014-2019, Analog Devices, Inc. All rights reserved.
 * Largely re-written for ADSP-SC5xx parts, but originally based on crt0.S:
 * Copyright (c) 1994-2009, Red Hat, Inc. All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 *
 * 1. Redistributions of source code must retain the above copyright notice,
 * this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright notice,
 * this list of conditions and the following disclaimer in the documentation
 * and/or other materials provided with the distribution.
 *
 * 3. Neither the name of the copyright holder nor the names of its
 * contributors may be used to endorse or promote products derived from this
 * software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 */

#include  <fatal_error_code.h>

#if defined(__ARM_ARCH_7A__)
	.arch armv7-a
#elif defined(__ARM_ARCH_7M__)
	.arch armv7-m
#else
#error Unsupported Architecture
#endif

	.text
	.syntax unified
#ifdef __SOFTFP__
    .fpu softvfp
#else
    .fpu neon-vfpv4
    .eabi_attribute 28, 1  /* Hardware FP Linkage */
#endif
#ifdef __thumb2__
	.thumb
.macro FUNC_START name
	.global \name
	.thumb_func
\name:
.endm
#else
	.code 32
.macro FUNC_START name
	.global	\name
\name:
.endm
#endif

	.align 	0

	FUNC_START	_mainCRTStartup
	FUNC_START	_start
#if defined(__ELF__) && !defined(__USING_SJLJ_EXCEPTIONS__)
/* Annotation for EABI unwinding tables.  */
	.fnstart
#endif

#ifdef __CYCLE_COUNT
/* Building the CRT with __CYCLE_COUNT defined will enable the performance
   monitor counters and then store the amount of time spent in start up
   code in the __crt_cycle_count global. */
    mov r0, #0xF
    movt r0, #0x8000
	mcr p15, 0, r0, c9, c12, 1   /* Enable counters in PMCNTENSET */
	mcr p15, 0, r0, c9, c12, 3   /* Clear overflows */
	mrc p15, 0, r0, c9, c12, 0   /* Enable and reset in PMCR */
	orr r0, r0, #0x17
	bic r0, r0, #8
	mcr p15, 0, r0, c9, c12, 0
#endif
    /* Make PA_06 High to beasure the boot time */
	movw r3, #0x4010
	movt r3, #0x3100
	ldr r2, [r3]
	movw r3, #0x4010
	movt r3, #0x3100
	orr r2, r2, #0x40
	str r2, [r3]


/* Disable Caches, MMU and branch prediction. */
	mrc p15, 0, r0, c1, c0, 0  /* Read CP15 Sys Control Reg */
	bic r0, r0, #(0x1 << 12)   /* Disable I-Cache */
	bic r0, r0, #(0x1 <<  2)   /* Disable D-Cache */
	bic r0, r0, #0x1           /* Disable MMU */
	bic r0, r0, #(0x1 << 11)   /* Disable branch prediction */
	mcr p15, 0, r0, c1, c0, 0  /* Write back to CP15 */

#ifndef __SOFTFP__
/* Enable vfp and neon. */
	mrc p15, 0, r0, c1, c0, 2
	orr r0, r0, #(0xF << 20)
	mcr p15, 0, r0, c1, c0, 2
	mov r0, #0x40000000
	vmsr FPEXC, r0
#endif

/* Set up stacks for all relevant modes. */
	mrs	r2, CPSR
	tst	r2, #0x0F           /* Test mode bits - if we're in User mode we are */
	beq	.Lskipstack         /* powerless, so we have to just move on. */
	movw	r3, #:lower16:__stack_fiq_start
	movt	r3, #:upper16:__stack_fiq_start
	mov r1, #0xD1
	msr     CPSR_c, r1      /* FIRQ mode, interrupts disabled. */
	mov 	sp, r3
	movw	r3, #:lower16:__stack_irq_start
	movt	r3, #:upper16:__stack_irq_start
	mov r1, #0xD2
	msr     CPSR_c, r1      /* IRQ mode, interrupts disabled. */
	mov	sp, r3
	movw	r3, #:lower16:__stack_abort_start
	movt	r3, #:upper16:__stack_abort_start
	mov r1, #0xD7
	msr     CPSR_c, r1      /* Abort mode, interrupts disabled. */
	mov	sp, r3
	movw	r3, #:lower16:__stack_undef_start
	movt	r3, #:upper16:__stack_undef_start
	mov r1, #0xDB
	msr     CPSR_c, r1      /* Undefined mode, interrupts disabled. */
	mov	sp, r3
	movw	r3, #:lower16:__stack_sup_start
	movt	r3, #:upper16:__stack_sup_start
	mov r1, #0xD3
	msr     CPSR_c, r1      /* Supervisor mode, interrupts disabled. */
	mov	sp, r3
	movw	r3, #:lower16:__stack_sys_start
	movt	r3, #:upper16:__stack_sys_start
	mov r1, #0xDF
	msr     CPSR_c, r1      /* System mode, interrupts disabled. */
	mov	sp, r3
.Lskipstack:
/* Setup a default stack-limit in-case the code has been */
/* compiled with "-mapcs-stack-check". */
	sub	sl, r3, #64 << 9

/* Zero the memory in the .bss section.  */
	movs a2, #0             /* Second arg: fill value.  */
	mov	fp, a2              /* Null frame pointer.  */
	mov	r7, a2              /* Null frame pointer for Thumb.  */

	ldr	a1, .Lbssstart      /* First arg: start of memory block.  */
	ldr	a3, .Lbssend
	subs	a3, a3, a1      /* Third arg: length of block.  */

	bl	memset
#ifndef __NO_MCAPI_INIT
/* Zero the MCAPI data section for common data. The ARM core needs to zero
   the section even if it's only used between the two SHARC cores. */

	movw	r0, #:lower16:__MCAPI_common_start
	movt	r0, #:upper16:__MCAPI_common_start
	movw	r3, #:lower16:__MCAPI_common_end
	movt	r3, #:upper16:__MCAPI_common_end
	subs	r2, r3, r0
	movs	r1, 0x0
	bl	memset
#endif

	bl adi_InitExceptionHandlers /* void function */

/* Call adi_cache_Init() API to configure caches. */
	bl adi_cache_Init       /* unconditionally returns success */

/* Call adi_mmu_Init() API to initialize page tables and the Memory-Management
   Unit (MMU). Branch to adi_fatal_error if an error is returned. */
call_adi_mmu_Init:
	bl adi_mmu_Init
	mov r1, #_AFE_S_GNUadi_mmu_Init_failure
	/* Load lr with the adi_mmu_Init call address for accurate 
	   adi_fatal_error PC output. */
	ldr lr, =call_adi_mmu_Init
	movs r2, r0             /* check return is 0 for ADI_MMU_SUCCESS */
	bne call_fatal_error    /* branch if adi_mmu_Init returned an error */

/* Initialize OSAL */
call_adi_osal_Init:
	bl adi_osal_Init
	mov r1, #_AFE_S_GNUadi_osal_Init_failure
	/* Load lr with the adi_osal_Init call address for accurate 
	   adi_fatal_error PC output. */
	ldr lr, =call_adi_osal_Init  
	movs r2, r0             /* check return is 0 for ADI_OSAL_SUCCESS */
	bne call_fatal_error    /* branch if adi_osal_Init returned an error */

/* Call the __libc_init_array function and arrange
   for __libc_fini_array to be called at program exit.  */
	ldr	r0, .Lfini
	bl atexit
	bl __libc_init_array

#ifdef __CYCLE_COUNT
/* Get the current value of PMCCNTR.  We reset it at the start, so the
   current value should be the time_spent_in_start_up - PMC_overhead. */
	mrc p15, 0, r2, c9, c13, 0
	movw    r3, #:lower16:__crt_cycle_count
	movt    r3, #:upper16:__crt_cycle_count
	str r2, [r3]
#endif

/* call main(), argc and argv set for no arguments. */
	movs	r0, #0
   movw	r1, #:lower16:__argv
   movt	r1, #:upper16:__argv
	bl main

call_exit:
	bl exit

/* Error - exit() should not return here. */
	ldr lr, =call_exit      /* load lr with adi_exit call address */
	mov r1, #_AFE_S_GNURetFromExit

/* Error handling using adi_fatal_error. */
call_fatal_error:
	/* Load GNURunTimeError general codes for init function error handling. */
#ifdef __thumb2__
	mov r0, #_AFE_G_GNURunTimeError
#else
	mov r0, #:lower16:_AFE_G_GNURunTimeError
	movt r0, #:upper16:_AFE_G_GNURunTimeError
#endif

	/* Branch without link so LR points to API call that returned the error. */
	b adi_fatal_error

/* For Thumb, constants must be after the code since only
   positive offsets are supported for PC relative addresses.  */
	.align 0

#if defined(__ELF__) && !defined(__USING_SJLJ_EXCEPTIONS__)
/* Protect against unhandled exceptions.  */
	.cantunwind
	.fnend
#endif
/* Include references to the stack objects so that they are linked into the application */
	.word __adi_stack_sys_object
	.word __adi_stack_sup_object
	.word __adi_stack_fiq_object
	.word __adi_stack_irq_object
	.word __adi_stack_abort_object
	.word __adi_stack_undef_object
.Lbssstart:
	.word	__bss_start__
.Lbssend:
	.word	__bss_end__
.Lfini:
	.word	__libc_fini_array
.data
.align 2
__argv:
	.word 0
#ifdef __CYCLE_COUNT
.global __crt_cycle_count
.data
.align 2
__crt_cycle_count:
	.word 0
#endif
