/*****************************************************************************
 * main.c
 *****************************************************************************/

#include "main.h"

#include <sys/platform.h>
#include <sys/adi_core.h>
#include "adi_initialize.h"

void adi_mmu_Init(void);

/*
 * The flash used on the EV-SC594-SOM EZ-kit are IS25LP512M and IS25LX256. Select any one of the macro below to select
 * different features/modes in boot ROM. SPI Master Boot will happen from IS25LP512M flash, while OSPI Master mode boots
 * from  IS25LX256, which is a octal flash that supports only single and octal mode.
 */

/*********************************USER CONFIGURABLE MACRO **********************************************/


/*Select one the macro below */
#define OSPI_SINGLE_MODE_STR
//#define OSPI_DUAL_MODE_STR
//#define OSPI_QUAD_MODE_STR
//#define OSPI_SINGLE_MODE_DTR
//#define OSPI_DUAL_MODE_DTR
//#define OSPI_QUAD_MODE_DTR
//#define OSPI_OCTAL_MODE_STR
//#define SPI_SINGLE_MODE
//#define SPI_DUAL_MODE
//#define SPI_QUAD_MODE
//#define UART_BOOT
//#define LP_BOOT
//#define SPISLAVE_BOOT

/*Select one the macro below */
//#define BLp
//#define BLx
//#define BLw

/*Select one the macro for the authentication mode */
//#define ECDSA224
//#define ECDSA256


/*Select one the macro below */
//#define SPLIT_BOOT
#ifdef ECDSA224
uint8_t PublicKey[] = {
    /* the Qx value */
    0x54, 0x79, 0xab, 0x3b,
    0xc4, 0x56, 0x08, 0x06,
    0x32, 0xe6, 0xf7, 0x83,
    0xed, 0xb9, 0xf6, 0xb9,
    0x68, 0x61, 0xea, 0xe7,
    0x36, 0x47, 0x8b, 0x91,
    0xd7, 0xfd, 0x46, 0xa4,
	0x00,0x00,0x00,0x00,

    /* the Qy value */
    0x97, 0xa5, 0x6d, 0x4b,
    0xe5, 0x45, 0x85, 0xa5,
    0x9a, 0x56, 0x8d, 0x83,
    0x09, 0xe3, 0x5f, 0x80,
    0xe1, 0x5e, 0x8c, 0x89,
    0x16, 0xb9, 0x57, 0x09,
    0xd5, 0x8e, 0xf6, 0xf1,
	0x00,0x00,0x00,0x00
};
#endif
#ifdef ECDSA256
uint8_t PublicKey[] = {
    /* the Qx value */
    0x31, 0x14, 0x7d, 0xd3,
    0x76, 0x37, 0xc3, 0xd3,
    0x91, 0x00, 0x26, 0x31,
    0xed, 0x29, 0x88, 0xd0,
    0x4b, 0xa6, 0xb5, 0x69,
    0xb3, 0xbf, 0x7b, 0x4c,
    0x6b, 0xd9, 0x0f, 0x10,
    0x3e, 0xd5, 0x44, 0xa5,

    /* the Qy value */
    0xc8, 0xe1, 0xab, 0x1e,
    0x83, 0x9e, 0x2d, 0xdb,
    0xcb, 0x72, 0x75, 0x02,
    0x9a, 0xba, 0x51, 0x3a,
    0x85, 0x4b, 0x3d, 0xe4,
    0x8c, 0x0f, 0x7f, 0x98,
    0xa8, 0x7c, 0x3a, 0x38,
    0x89, 0x8b, 0x6a, 0x82

};
#endif
#ifdef BLx
uint8_t AESKey[] = {
		0x3a,0x65,0x26,0x30,0x97,0x55,0x27,0x7e,0xd2,0x3d,0x61,0xaf,0x6e,0x17,0x8f,0xa3


};
#endif
#ifdef BLw
uint8_t AESKey[] = {
		0x3a,0x65,0x26,0x30,0x97,0x55,0x27,0x7e,0xd2,0x3d,0x61,0xaf,0x6e,0x17,0x8f,0xa3

};
uint8_t AESwrapKey[]={
		0x3a,0x65,0x26,0x30,0x97,0x55,0x27,0x7e,0xd2,0x3d,0x61,0xaf,0x6e,0x17,0x8f,0xa3

};
#endif


int main()
{

	adi_initComponents();

	/*SPU Secure Read/Write access launched by MDMA0_SRC*/
	*pREG_SPU0_SECUREP146= 0x3;
	/*SPU Secure Read/Write access launched by MDMA0_DST*/
	*pREG_SPU0_SECUREP147= 0x3;

	/*Toggle GPIO for initial point for boot time measurement.boot application start up
	 *  should have the next toggle on the GPIO pin to capture the end point .*/
	*pREG_PORTA_FER_CLR |= BITM_PORT_FER_PX6;
	*pREG_PORTA_DIR_SET |= BITM_PORT_FER_PX6;
	*pREG_PORTA_DATA_CLR |= BITM_PORT_FER_PX6;

#ifdef OSPI_SINGLE_MODE_STR
#ifdef SPLIT_BOOT
	adi_rom_Boot(0x60400000,BITM_ROM_BFLAG_HOOK|BITM_ROM_BFLAG_RETURN,0,&ROM_BOOT_HOOK_FUNC1,0x30210048);/*No-Auto Detection - Single STR BCODE 1: 0x30210348 */
	adi_rom_Boot(0x60000000,BITM_ROM_BFLAG_HOOK,0,&ROM_BOOT_HOOK_FUNC1,0x30210048);/*No-Auto Detection - Single STR BCODE 1: 0x30210348 */
#else

	adi_rom_Boot(0x60000000,BITM_ROM_BFLAG_HOOK,0,&ROM_BOOT_HOOK_FUNC1,0x30210048);/*No-Auto Detection - Single STR BCODE 1: 0x30210048 */
#endif
#elif defined  OSPI_DUAL_MODE_STR
#ifdef SPLIT_BOOT
	adi_rom_Boot(0x60400000,BITM_ROM_BFLAG_RETURN|BITM_ROM_BFLAG_HOOK,0,&ROM_BOOT_HOOK_FUNC1,0x31230048);/*No-Auto Detection - Dual STR BCODE 3 : 0x31230348 */
	adi_rom_Boot(0x60000000,BITM_ROM_BFLAG_HOOK,0,&ROM_BOOT_HOOK_FUNC1,0x31230048);/*No-Auto Detection - Dual STR BCODE 3 : 0x31230348 */
#else

	adi_rom_Boot(0x60000000,BITM_ROM_BFLAG_HOOK,0,&ROM_BOOT_HOOK_FUNC1,0x31230048);/*No-Auto Detection - Dual STR BCODE 3 : 0x31230048 */
#endif
#elif defined OSPI_QUAD_MODE_STR
#ifdef SPLIT_BOOT
	adi_rom_Boot(0x60400000,BITM_ROM_BFLAG_RETURN|BITM_ROM_BFLAG_HOOK,0,&ROM_BOOT_HOOK_FUNC1,0x31A50048);/*No-Auto Detection - Quad STR BCODE 5: 0x31A50348 */
	adi_rom_Boot(0x60000000,BITM_ROM_BFLAG_HOOK,0,&ROM_BOOT_HOOK_FUNC1,0x31A50048);/*No-Auto Detection - Quad STR BCODE 5: 0x31A50348 */
#else

	adi_rom_Boot(0x60000000,BITM_ROM_BFLAG_HOOK,0,&ROM_BOOT_HOOK_FUNC1,0x31A50048);/*No-Auto Detection - Quad STR BCODE 5: 0x31A50048 */
#endif
#elif defined OSPI_SINGLE_MODE_DTR
#ifdef SPLIT_BOOT
	adi_rom_Boot(0x60400000,BITM_ROM_BFLAG_RETURN|BITM_ROM_BFLAG_HOOK,0,&ROM_BOOT_HOOK_FUNC1,0x72280048);/*No-Auto Detection - Quad DTR BCODE 8: 0x72280348 */
	adi_rom_Boot(0x60000000,BITM_ROM_BFLAG_HOOK,0,&ROM_BOOT_HOOK_FUNC1,0x72280048);/*No-Auto Detection - Quad DTR BCODE 8: 0x72280348 */
#else

	adi_rom_Boot(0x60000000,BITM_ROM_BFLAG_HOOK,0,&ROM_BOOT_HOOK_FUNC1,0x72280048);/*No-Auto Detection - Quad DTR BCODE 8: 0x72280348 */
#endif
#elif defined OSPI_DUAL_MODE_DTR
#ifdef SPLIT_BOOT
	adi_rom_Boot(0x60400000,BITM_ROM_BFLAG_RETURN|BITM_ROM_BFLAG_HOOK,0,&ROM_BOOT_HOOK_FUNC1,0x712A0048);/*No-Auto Detection - Quad DTR BCODE A: 0x712A0348 */
	adi_rom_Boot(0x60000000,BITM_ROM_BFLAG_HOOK,0,&ROM_BOOT_HOOK_FUNC1,0x712A0048);/*No-Auto Detection - Quad DTR BCODE A: 0x712A0348 */
#else

	adi_rom_Boot(0x60000000,BITM_ROM_BFLAG_HOOK,0,&ROM_BOOT_HOOK_FUNC1,0x712A0048);/*No-Auto Detection - Quad DTR BCODE A: 0x712A0348 */
#endif
#elif defined OSPI_QUAD_MODE_DTR
#ifdef SPLIT_BOOT
	adi_rom_Boot(0x60400000,BITM_ROM_BFLAG_RETURN|BITM_ROM_BFLAG_HOOK,0,&ROM_BOOT_HOOK_FUNC1,0x71AC0048);/*No-Auto Detection - Quad DTR BCODE C: 0x71AC0348 */
	adi_rom_Boot(0x60000000,BITM_ROM_BFLAG_HOOK,0,&ROM_BOOT_HOOK_FUNC1,0x71AC0048);/*No-Auto Detection - Quad DTR BCODE C: 0x71AC0348 */
#else

	adi_rom_Boot(0x60000000,BITM_ROM_BFLAG_HOOK,0,&ROM_BOOT_HOOK_FUNC1,0x71AC0048);/*No-Auto Detection - Quad DTR BCODE C: 0x71AC0348 */
#endif
#elif defined OSPI_OCTAL_MODE_STR
#ifdef SPLIT_BOOT
	adi_rom_Boot(0x60400000,BITM_ROM_BFLAG_RETURN|BITM_ROM_BFLAG_HOOK,0,&ROM_BOOT_HOOK_FUNC1,0x00000008);/*OSPI mode for the Octal STR mode */
	adi_rom_Boot(0x60000000,BITM_ROM_BFLAG_HOOK,0,&ROM_BOOT_HOOK_FUNC1,0x00000008);/*OSPI mode for the Octal STR mode */
#else

	adi_rom_Boot(0x60000000,BITM_ROM_BFLAG_HOOK,0,&ROM_BOOT_HOOK_FUNC1,0x00000008);/*OSPI mode for the Octal STR mode */
#endif
#elif defined SPI_SINGLE_MODE
#ifdef SPLIT_BOOT
	adi_rom_Boot(0x60400000,BITM_ROM_BFLAG_RETURN | BITM_ROM_BFLAG_HOOK,0,&ROM_BOOT_HOOK_FUNC1,0x10620247);/*No-Auto Detection - Single STR FAST mode BCODE 2: 0x20210247 */
	adi_rom_Boot(0x60000000,BITM_ROM_BFLAG_HOOK,0,&ROM_BOOT_HOOK_FUNC1,0x10620247);/*No-Auto Detection - Single STR FAST mode BCODE 2: 0x10620247 */

#else

	adi_rom_Boot(0x60000000,BITM_ROM_BFLAG_HOOK,0,&ROM_BOOT_HOOK_FUNC1,0x207);//0x10620247);/*No-Auto Detection - Single STR FAST mode BCODE 2: 0x20210247 */

#endif
#elif defined SPI_DUAL_MODE
#ifdef SPLIT_BOOT
	adi_rom_Boot(0x60400000,BITM_ROM_BFLAG_RETURN|BITM_ROM_BFLAG_HOOK,0,&ROM_BOOT_HOOK_FUNC1,0x10650247);/*No-Auto Detection - Dual STR BCODE 5: 0x10650247 */
	adi_rom_Boot(0x60000000,BITM_ROM_BFLAG_HOOK,0,&ROM_BOOT_HOOK_FUNC1,0x10650247);/*No-Auto Detection - Dual STR BCODE 5: 0x10650247 */
#else

	adi_rom_Boot(0x60000000,BITM_ROM_BFLAG_HOOK,0,&ROM_BOOT_HOOK_FUNC1,0x10650247);/*No-Auto Detection - Dual STR BCODE 1: 0x20650247 */
#endif
#elif SPI_QUAD_MODE
#ifdef SPLIT_BOOT
	adi_rom_Boot(0x60400000,BITM_ROM_BFLAG_RETURN|BITM_ROM_BFLAG_HOOK,0,&ROM_BOOT_HOOK_FUNC1,0x10E90247);/*No-Auto Detection - Quad STR BCODE 9: 0x10E90247 */
	adi_rom_Boot(0x60000000,BITM_ROM_BFLAG_HOOK,0,&ROM_BOOT_HOOK_FUNC1,0x10E90247);/*No-Auto Detection - Quad STR BCODE 9: 0x10E90247 */
#else

	adi_rom_Boot(0x60000000,BITM_ROM_BFLAG_HOOK,0,&ROM_BOOT_HOOK_FUNC1,0x10E90247);/*No-Auto Detection - Quad STR BCODE 1: 0x20E90247 */
#endif
#endif

#ifdef UART_BOOT
	*pREG_SPU0_SECUREP79= 0x3;
	adi_rom_Boot(0x00000000,BITM_ROM_BFLAG_HOOK,0,&ROM_BOOT_HOOK_FUNC1,0x3);/*UART Slave Boot*/

#endif

#ifdef LP_BOOT
	*pREG_SPU0_SECUREP5 = 0x3;
	*pREG_SPU0_SECUREP6 = 0x3;
	uint32_t BootCmd=0;
	BootCmd = (0<<BITP_ROM_BCMD_DEVENUM)|(1<<BITP_ROM_BCMD_LINKPORT_HOST)|(4<<BITP_ROM_BCMD_DEVICE);
	adi_rom_Boot(0x00000000,BITM_ROM_BFLAG_HOOK,0,&ROM_BOOT_HOOK_FUNC1,BootCmd); /*LP Slave Boot*/
#endif


#ifdef	SPISLAVE_BOOT
	*pREG_SPU0_SECUREP97 = 0x3;
	*pREG_SPU0_SECUREP98 = 0x3;
	uint32_t BootCmd=0;
	BootCmd = (0<<BITP_ROM_BCMD_SPIS_BCODE)|(2<<BITP_ROM_BCMD_DEVENUM)|(1<<BITP_ROM_BCMD_SPIS_NOAUTO)|(1<<BITP_ROM_BCMD_SPIS_HOST)|(2<<BITP_ROM_BCMD_SPIS_DEVICE);
	adi_rom_Boot(0x00000000,0,0,0,BootCmd); /*SPI Slave Boot*/

#endif


	return 0;
}

uint32_t ROM_BOOT_HOOK_FUNC1(ADI_ROM_BOOT_CONFIG * pBootConfig, ROM_HOOK_CALL_CAUSE cause)
{
	if (cause == ROM_HOOK_REG_COMPLETE)
	{
		asm("NOP;");
	}
	if (cause == ROM_HOOK_CALL_INIT_COMPLETE)
	{
		asm("NOP;");
		pBootConfig->keyType = ADI_ROM_CUSTOM_SECURITY; /*Tell ROM that we want to use custom key*/
		//pBootConfig->bootType = ADI_ROM_SECURE_BOOT;/*Secure boot*/

		uint8_t *pPublicKey  = &pBootConfig->publicKey;
		uint8_t *pAESKey     = (uint8_t *)&pBootConfig->aesKey;

		int i=0;
#if defined(BLp)||defined(BLx)||defined(BLw)

		for(i=0;i<64;i++)
		{
			pPublicKey[i] = PublicKey[i];
		}



#endif

		#ifdef BLx
		for(i=0;i<16;i++)
		{
			pAESKey[i] = AESKey[i];
		}

		#endif
		#ifdef BLw
		for(i=0;i<16;i++)
		{
			pAESKey[i] = AESwrapKey[i];
		}
		#endif
	}
	if (cause == ROM_HOOK_CALL_CONFIG_COMPLETE)
	{
		asm("NOP;");
#ifdef OSPI_OCTAL_MODE_STR
		*pREG_PORTC_FER |= (BITM_PORT_FER_PX8 |BITM_PORT_FER_PX9| BITM_PORT_FER_PX10 | BITM_PORT_FER_PX11 | BITM_PORT_FER_PX12| BITM_PORT_FER_PX13 |
	    			   BITM_PORT_FER_PX14 | BITM_PORT_FER_PX15);
		*pREG_PORTD_FER |= (BITM_PORT_FER_PX0 |BITM_PORT_FER_PX1);

		*pREG_OSPI0_CTL &= (~BITM_OSPI_CTL_BAUD);
		*pREG_OSPI0_CTL |= (0xF <<BITP_OSPI_CTL_BAUD);

		*pREG_OSPI0_DRICTL = 0x803008B;
#endif

#if defined(OSPI_SINGLE_MODE_DTR)||defined(OSPI_DUAL_MODE_DTR)||defined(OSPI_QUAD_MODE_DTR)
		/*Increasing the baud rate and updating RDCR for operation at max freq of 62.5MHz with default preload code */
		*pREG_OSPI0_CTL &= ~BITM_OSPI_CTL_BAUD;
		*pREG_OSPI0_CTL |= 0x3<<BITP_OSPI_CTL_BAUD;
		*pREG_OSPI0_RDC &= ~BITM_OSPI_RDC_DLYRD;
		*pREG_OSPI0_RDC |= (6 << 1);
#endif
	}

	return 0;
}




